﻿#include "StdAfx.h"
#include "SearchWnd.h"
#include "UIMenu.h"
#include "main_frame.hpp"
#include "NeMessengerDlg.h"

DUI_BEGIN_MESSAGE_MAP(CSearchWnd, WindowImplBase, FALSE)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_WINDOWINIT, OnPrepare)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_ITEMDBCLICK, _OnCtrlElementUIDbClick)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_ITEMCLICK, _OnCtrlElementUIItemClick)

    DUI_ON_CLICK_CTRNAME(kOrgBtnIPT, _OnCtrlOrgBtnIPTClick)
    DUI_ON_CLICK_CTRNAME(kFavorite, _OnCtrlFavoriteClick)
DUI_END_MESSAGE_MAP()

DUI_BEGIN_CUSTOM_MSG_MAP(CSearchWnd)
DUI_END_CUSTOM_MSG_MAP()

CSearchWnd::CSearchWnd(HWND hWnd, CRect rc)
{
	m_rcClient = rc;
	m_bEnableClose = TRUE;
}

CSearchWnd::~CSearchWnd()
{
	
}

LPCTSTR CSearchWnd::GetWindowClassName() const
{
	return _T("SearchWindow");
}

void CSearchWnd::OnFinalMessage(HWND hWnd)
{
	CSearchWndList* pFriendsList = static_cast<CSearchWndList*>(m_PaintManager.FindControl(kSearchList));
	if (pFriendsList != NULL)
	{
		pFriendsList->RemoveAll();
	}
	WindowImplBase::OnFinalMessage(hWnd);
	if (GET_MESSENGER_DLG()->GetMainFrame() != NULL)
	{
		::PostMessage(GET_MESSENGER_DLG()->GetMainFrame()->GetHWND(), MSG_CLOSE_MAIN_SEARCH, 0, 0);
	}
	delete this;
}

NTIPTStatusType CSearchWnd::GetMemberIPTStatus(LPCWSTR strUserID)
{
	NTIPTStatusType iptStatusType = iptstIDLE;
	const IGTBuddyUser* pBuddyUser = g_pMessengerActor->GetBuddyInfo(strUserID);
	if (pBuddyUser != NULL)
	{
		iptStatusType = pBuddyUser->GetIPTStatus();
		return iptStatusType;
	}
	const IGTBuddyUser* pPeristUser = g_pMessengerActor->GetBuddyInfoFromPersist(strUserID);
	if (pPeristUser != NULL)
	{
		iptStatusType = pPeristUser->GetIPTStatus();
		return iptStatusType;
	}
	iptStatusType = GET_MESSENGER_DLG()->GetWorkGroupController()->m_pModel->GetMemberIPTStatus(strUserID);
	return iptStatusType;
}

void CSearchWnd::MakeIPTMenu( NTIPTStatusType iptMyStatus, NTIPTStatusType iptStatusOfBuddy, const NE_MEMBER_DATA* pMembereData, CMenu* pPopup )
{
	CString strCorpTel = pMembereData->strIncoTel.c_str();
	CString strCellPhone = pMembereData->strCellPhone.c_str();
	CString strExtensionTel = pMembereData->strExtesion.c_str();

	//pPopup->InsertMenu(ID_MENU_CONFER_CALL, MF_BYCOMMAND, MF_SEPARATOR );
	if (!(iptMyStatus == iptstIDLE || iptMyStatus == iptstDND || iptMyStatus == iptstFWD) || iptStatusOfBuddy != iptstIDLE || strExtensionTel.IsEmpty() || !GET_CONFIG()->EnableFunction(_T("SR_08")))
	{
		pPopup->EnableMenuItem(ID_MENU_IPT_EXT_PHONE, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
	}

	if (!(iptMyStatus == iptstIDLE || iptMyStatus == iptstDND || iptMyStatus == iptstFWD) || iptStatusOfBuddy != iptstIDLE || strCorpTel.IsEmpty() || !GET_CONFIG()->EnableFunction(_T("SR_05")))
	{
		pPopup->EnableMenuItem(ID_MENU_IPT_CORP_PHONE, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
	}

	if (!(iptMyStatus == iptstIDLE || iptMyStatus == iptstDND || iptMyStatus == iptstFWD) || strCellPhone.IsEmpty() || !GET_CONFIG()->EnableFunction(_T("SR_07")))
	{	
		pPopup->EnableMenuItem(ID_MENU_IPT_CELLPHONE, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
	}
	if(iptMyStatus != iptstACTIVE || strExtensionTel.IsEmpty() || !GET_CONFIG()->EnableFunction(_T("SR_08")))
	{
		pPopup->EnableMenuItem(ID_MENU_IPT_CONFER, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
		pPopup->EnableMenuItem(ID_MENU_IPT_TRANSFER, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
	}

	if (!GET_CONFIG()->EnableFunction(_T("SR_08")))
	{
		pPopup->DeleteMenu(ID_MENU_IPT_CONFER, MF_BYCOMMAND);
		pPopup->DeleteMenu(ID_MENU_IPT_TRANSFER, MF_BYCOMMAND);
		pPopup->DeleteMenu(ID_MENU_IPT_EXT_PHONE, MF_BYCOMMAND);
	}
	if (!GET_CONFIG()->EnableFunction(_T("SR_05")))
	{
		pPopup->DeleteMenu(ID_MENU_IPT_CORP_PHONE, MF_BYCOMMAND);
	}
	if (!GET_CONFIG()->EnableFunction(_T("SR_07")))
	{
		pPopup->DeleteMenu(ID_MENU_IPT_CELLPHONE, MF_BYCOMMAND);
	}
}

void CSearchWnd::OnMenuIPT(const NE_MEMBER_DATA* pMembereData, WPARAM wParam)
{
	int nID = (int)wParam;
	CString strTelNo = _T("");

	switch (nID)
	{
	case ID_MENU_COMPANY_PHONE:
	case ID_MENU_IPT_CORP_PHONE:
		strTelNo = pMembereData->strIncoTel.c_str();
		break;

	case ID_MENU_CELL_PHONE:
	case ID_MENU_IPT_CELLPHONE:
		strTelNo = pMembereData->strCellPhone.c_str();
		break;

	case ID_MENU_EXT_PHONE:
	case ID_MENU_IPT_EXT_PHONE:
		strTelNo = pMembereData->strExtesion.c_str();
		break;
	}

	CallToIPT(*pMembereData, strTelNo);
}

void CSearchWnd::CallToIPT(NE_MEMBER_DATA userData, CString strTel)
{
	if (strTel.IsEmpty())
		return;

	CDialPadManager* pManager = CDialPadManager::GetDialpadManager();

	if (pManager)
		pManager->MakeCall(userData, strTel);
}


void CSearchWnd::_OnCtrlOrgBtnIPTClick(TNotifyUI& msg)
{
    if(GET_APP()->IsAvailableIPT())
    {
        Node* node = (Node*)msg.pSender->GetTag();
        if (node != NULL)
        {
            NTIPTStatusType iptMyStatus = g_pMessengerActor->GetMyIptStatus();
            NTIPTStatusType iptStatusOfBuddy = GetMemberIPTStatus(node->data().value);
            const NE_MEMBER_DATA MembereData = GET_MESSENGER_DLG()->GetWorkGroupController()->m_pModel->GetMember(node->data().value);
            CMenu menuList;
            menuList.LoadMenu(IDR_MEMU_COMMON_MENU);
            CMenu* pPopup = menuList.GetSubMenu(7);
            MakeIPTMenu(iptMyStatus, iptStatusOfBuddy, &MembereData, pPopup);

            if( pPopup != NULL )
            {
                CPoint point(msg.ptMouse.x, msg.ptMouse.y);
                ::ClientToScreen(m_hWnd, &point);
                int nCmdID = pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RETURNCMD | TPM_RIGHTBUTTON, point.x, point.y, CWnd::FromHandle(m_hWnd));
                OnMenuIPT(&MembereData, nCmdID);
            }
        }
    }
}

void CSearchWnd::_OnCtrlFavoriteClick(TNotifyUI& msg)
{
    vector<Node*> vecSelID;

    Node* node = (Node*)msg.pSender->GetTag();
    vecSelID.push_back(node);
    GET_MESSENGER_DLG()->GetMainFrame()->SetSelectItems(vecSelID);
    CPoint point = msg.ptMouse;
    ClientToScreen(m_hWnd, &point);
    GET_MESSENGER_DLG()->GetMainFrame()->SetCurSelCtrl(node);
    GET_MESSENGER_DLG()->GetMainFrame()->OnBtnFavorite(point, (CButtonUI*)msg.pSender);
}

void CSearchWnd::_OnCtrlElementUIDbClick(TNotifyUI& msg)
{
    //ASSERT(_tcsicmp(msg.pSender->GetClass(), _T("ListContainerElementUI")) == 0);

    CSearchWndList* pOrgTree = static_cast<CSearchWndList*>(m_PaintManager.FindControl(kSearchList));
    if ((pOrgTree != NULL) &&  pOrgTree->GetItemIndex(msg.pSender) != -1)
    {
        if (_tcsicmp(msg.pSender->GetClass(), _T("ListContainerElementUI")) == 0)
        {
            Node* node = (Node*)msg.pSender->GetTag();
            if (!node->folder())
            {
                std::vector<NE_MEMBER_DATA> memberVector;
                CString strID = node->data().value;
                const NE_MEMBER_DATA data = GET_MESSENGER_DLG()->GetWorkGroupController()->m_pModel->GetMember(strID);
                GTUserStatusType eUserStatus = GET_MESSENGER_DLG()->GetWorkGroupController()->m_pModel->GetMemberStatus(strID);
                if(eUserStatus != ustOFFLINE && !GET_MESSENGER_DLG()->GetWorkGroupController()->IsMyID(strID))
                {
                    memberVector.push_back(data);
                    GET_APP()->m_pChatDialogManager->CreateMemberChat(memberVector);
                }
            }
        }
    }
}

void CSearchWnd::_OnCtrlElementUIItemClick(TNotifyUI& msg)
{
    CSearchWndList* pOrgTree = static_cast<CSearchWndList*>(m_PaintManager.FindControl(kSearchList));
    if ((pOrgTree != NULL) &&  pOrgTree->GetItemIndex(msg.pSender) != -1)
    {
        if (_tcsicmp(msg.pSender->GetClass(), _T("ListContainerElementUI")) == 0)
        {
            Node* node = (Node*)msg.pSender->GetTag();
            if (msg.wParam == UIEVENT_RBUTTONUP)
            {
                if (!node->folder())
                {
                    CPoint point = msg.ptMouse;
                    ClientToScreen(m_hWnd, &point);
                    if (GET_MESSENGER_DLG()->GetMainFrame() != NULL)
                    {
                        CRect rcItem = msg.pSender->GetPos();
                        CRect rcSearchWnd = GET_MESSENGER_DLG()->GetMainFrame()->GetSearchWndRect();
                        rcItem.left = rcSearchWnd.left;
                        rcItem.top += rcSearchWnd.top;
                        rcItem.right = rcSearchWnd.right;
                        rcItem.bottom += rcSearchWnd.top;
                        GET_MESSENGER_DLG()->GetMainFrame()->SetCurSelCtrl(node);

                        vector<Node*> vecSelID;

                        list<int> listSelRows = pOrgTree->GetSelectedItems();
                        if (listSelRows.size() > 0)
                        {
                            list<int>::iterator it = listSelRows.begin();
                            for (; it!=listSelRows.end(); ++it)
                            {
                                vecSelID.push_back(pOrgTree->GetItemNode(*it));
                            }
                        }

                        GET_MESSENGER_DLG()->GetMainFrame()->SetSelectItems(vecSelID);
                        GET_MESSENGER_DLG()->GetMainFrame()->PopupMenuView(point, IDR_MEMU_COMMON_MENU, LIST_POPUP_MENU);
                    }
                }

            }
        }
    }
}

LRESULT CSearchWnd::OnKeyDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    if ((TCHAR)wParam == VK_ESCAPE)
    {
        GET_MESSENGER_DLG()->GetMainFrame()->CloseSearchWnd();
        bHandled = TRUE;
    }
    else
    {
        bHandled = FALSE;
    }
    return 0; 
}

LRESULT CSearchWnd::OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    if (GET_MESSENGER_DLG()->GetMainFrame() != NULL)
    {
        CRect rcSearch(0, 0, 0, 0);
        GetWindowRect(m_hWnd, rcSearch);
        CRect rcClient(0, 0, 0, 0);
        GetWindowRect(GET_MESSENGER_DLG()->GetMainFrame()->GetHWND(), rcClient);
        int nTopDistance = rcSearch.top-rcClient.top;
        if (206 > nTopDistance && nTopDistance > 0)
        {
            rcSearch = GET_MESSENGER_DLG()->GetMainFrame()->GetSearchWndRect();
            SetClientRect(rcSearch);
        }
        else
        {
            GET_MESSENGER_DLG()->GetMainFrame()->SetSearchWndHeight(rcSearch.Height());
        }
    }

	bHandled = FALSE;

    return 0;
}

CControlUI* CSearchWnd::CreateControl(LPCTSTR pstrClass)
{
	if (_tcsicmp(pstrClass, _T("SearchWndList")) == 0)
	{
		return new CSearchWndList(m_PaintManager, m_hWnd);
	}

	return NULL;
}

void CSearchWnd::SetClientRect(CRect rc)
{
	m_rcClient = rc;
	MoveWindow(GetHWND(), m_rcClient.left, m_rcClient.top, m_rcClient.Width(), m_rcClient.Height(), TRUE);
}

void CSearchWnd::InitWindow()
{
	MoveWindow(GetHWND(), m_rcClient.left, m_rcClient.top, m_rcClient.Width(), m_rcClient.Height(), TRUE);
}

void CSearchWnd::OnPrepare(TNotifyUI& msg)
{
	//UpdateFriendsList();
}

CDuiString CSearchWnd::GetSkinFile()
{
	return _T("searchWnd.xml");
}

CDuiString CSearchWnd::GetSkinFolder()
{
	return _T("skin\\nTalkRes");
}

BOOL CSearchWnd::IsEnableClose()
{
	return m_bEnableClose;
}

void CSearchWnd::EnableCloseWindow(BOOL bEnable)
{
	m_bEnableClose = bEnable;
}

LRESULT CSearchWnd::OnKillFocus(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	CPoint pt(0, 0);
	GetCursorPos(&pt);
	CRect rcClient(0, 0, 0, 0);
	GetWindowRect(m_hWnd, rcClient);
	CRect rcSearchBtn(rcClient.left + 3, rcClient.bottom, rcClient.left + 25, rcClient.bottom + 30);
	CRect rcProfile(0, 0, 0, 0);
	BOOL bOnProfile = FALSE;
	if (g_pProfileWnd != NULL && IsWindow(g_pProfileWnd->GetHWND()) && g_pProfileWnd->GetParentHwnd() == m_hWnd)
	{
		GetWindowRect(g_pProfileWnd->GetHWND(), rcProfile);
		if (rcProfile.PtInRect(pt))
		{
			bOnProfile = TRUE;
		}
	}
	if (!rcClient.PtInRect(pt) && !rcSearchBtn.PtInRect(pt) && !bOnProfile)
	{
		if (GET_MESSENGER_DLG()->GetMainFrame() != NULL && IsWindow(GET_MESSENGER_DLG()->GetMainFrame()->GetHWND()) && GET_MESSENGER_DLG()->GetMainFrame()->IsShowMainSearchWnd())
		{
			GET_MESSENGER_DLG()->GetMainFrame()->CloseSearchWnd();
		}
	}

	return 0;
}

CSearchWndList* CSearchWnd::GetFriendsCtrl()
{
	CSearchWndList* pFriendsList = static_cast<CSearchWndList*>(m_PaintManager.FindControl(kSearchList));
	return pFriendsList;
}

void CSearchWnd::UpdateFriendsList()
{
	
}

LRESULT CSearchWnd::OnNcHitTest(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	int ret = __super::OnNcHitTest(uMsg, wParam, lParam, bHandled);
	if(HTTOP != ret)
	{
		ret = HTCLIENT;
	}
	return ret;
}